#ifndef NB_BIOMETRICS_CONTEXT_HPP
#define NB_BIOMETRICS_CONTEXT_HPP

#include <NBBiometricsTypes.hpp>
#include <NBBiometricsTemplate.hpp>
#include <iterator>
#include <vector>

namespace NextBiometrics
{
    namespace Biometrics
    {
#include <NBBiometricsContext.h>

        class NBBiometricsCreateEnrollTemplateResult
        {
            friend class NBBiometricsContext;

            NBBiometricsStatus m_eStatus;
            NBBiometricsTemplate m_template;

            NBBiometricsCreateEnrollTemplateResult(NBBiometricsStatus eStatus)
                : NBBiometricsCreateEnrollTemplateResult(eStatus, NBBiometricsTemplate(NULL, 0, 0))
            {
            }

            NBBiometricsCreateEnrollTemplateResult(NBBiometricsStatus eStatus, const NBBiometricsTemplate& biometricsTemplate)
                : m_eStatus(eStatus), m_template(biometricsTemplate)
            {
            }

            NBBiometricsCreateEnrollTemplateResult(NBBiometricsStatus eStatus, const NBByte * pData, NBSizeType stDataSize, NBSizeType stActualSize)
                : m_eStatus(eStatus), m_template(pData, stDataSize, stActualSize)
            {
            }
        public:
            NBBiometricsStatus GetStatus() const
            {
                return m_eStatus;
            }

            const NBBiometricsTemplate& GetTemplate() const
            {
                return m_template;
            }
        };

        class NBBiometricsExtractResult
        {
            friend class NBBiometricsContext;

            NBBiometricsStatus m_eStatus;
            NBBiometricsTemplate m_template;

            NBBiometricsExtractResult(NBBiometricsStatus eStatus)
                : NBBiometricsExtractResult(eStatus, NBBiometricsTemplate(NULL, 0, 0))
            {
            }

            NBBiometricsExtractResult(NBBiometricsStatus eStatus, const NBBiometricsTemplate& biometricsTemplate)
                : m_eStatus(eStatus), m_template(biometricsTemplate)
            {
            }

            NBBiometricsExtractResult(NBBiometricsStatus eStatus, const NBByte * pData, NBSizeType stDataSize, NBSizeType stActualSize)
                : m_eStatus(eStatus), m_template(pData, stDataSize, stActualSize)
            {
            }
        public:
            NBBiometricsStatus GetStatus() const
            {
                return m_eStatus;
            }

            const NBBiometricsTemplate& GetTemplate() const
            {
                return m_template;
            }
        };

        class NBBiometricsVerifyResult
        {
            friend class NBBiometricsContext;

            NBBiometricsStatus m_eStatus;
            NBInt m_iScore;

            NBBiometricsVerifyResult(NBBiometricsStatus eStatus, NBInt iScore)
                : m_eStatus(eStatus), m_iScore(iScore)
            {
            }

            NBBiometricsVerifyResult(NBBiometricsStatus eStatus, const NBBiometricsVerifyResultDetails& sVerifyResultDetails)
                : NBBiometricsVerifyResult(eStatus, sVerifyResultDetails.iScore)
            {
            }
        public:
            NBBiometricsStatus GetStatus() const
            {
                return m_eStatus;
            }

            NBInt GetScore() const
            {
                return m_iScore;
            }
        };

        class NBBiometricsIdentifyResult
        {
            friend class NBBiometricsContext;

            NBBiometricsStatus m_eStatus;
            const NBVoid * m_pTemplateId;
            NBInt m_iScore;

            NBBiometricsIdentifyResult(NBBiometricsStatus eStatus, const NBVoid * pTemplateId, NBInt iScore)
                : m_eStatus(eStatus), m_pTemplateId(pTemplateId), m_iScore(iScore)
            {
            }

            NBBiometricsIdentifyResult(NBBiometricsStatus eStatus, const NBBiometricsIdentifyResultDetails& sIdentifyResultDetails)
                : NBBiometricsIdentifyResult(eStatus, sIdentifyResultDetails.pTemplateId, sIdentifyResultDetails.iScore)
            {
            }
        public:
            NBBiometricsStatus GetStatus() const
            {
                return m_eStatus;
            }

            NBInt GetScore() const
            {
                return m_iScore;
            }

            const NBVoid * GetTemplateId() const
            {
                return m_pTemplateId;
            }
        };

        class NBBiometricsScanPreviewEventArgs;
        typedef void(*NBBiometricsScanPreviewEvent)(NBBiometricsScanPreviewEventArgs &args);

        class NBBiometricsContext
        {
            friend class NBBiometricsTemplate;
            friend class NBBiometricsScanPreviewEventArgs;
            friend class NBBiometricsScanPreviewEventHandler;

            class NBBiometricsScanPreviewEventHandler
            {
                friend class NBBiometricsContext;

                NBBiometricsScanPreviewEvent m_callback;
                void * m_pParam;

                NBBiometricsScanPreviewEventHandler(NBBiometricsScanPreviewEvent callback, void * pParam)
                    : m_callback(callback), m_pParam(pParam)
                {
                }

                static NBResult NB_API NativeCallback(const HNBBiometricsContext hContext, NBBiometricsStatus eBiometricsStatus, const HNBDevice hDevice, NBDeviceScanStatus * peStatus, const NBDeviceScanFormatInfo * psScanFormatInfo, const NBByte * pBuffer, NBSizeType stBufferSize, const NBDeviceScanPreviewDetails * psDetails, void * pParam);
            };

            template<typename T>
            class NBBiometricsTemplateIteratorEventHandler
            {
                friend class NBBiometricsContext;

                T m_items;
                typename T::const_iterator m_iteratorBegin;
                typename T::const_iterator m_iteratorEnd;

                NBBiometricsTemplateIteratorEventHandler(T items)
                    : m_items(items)
                {
                }

                static NBResult NB_API InitNativeCallback(NBVoid * pContext)
                {
                    NBResult result = NBErrors::NB_OK;
                    try
                    {
                        NBBiometricsTemplateIteratorEventHandler * pHandler = reinterpret_cast<NBBiometricsTemplateIteratorEventHandler*>(pContext);
                        pHandler->m_iteratorBegin = pHandler->m_items.begin();
                        pHandler->m_iteratorEnd = pHandler->m_items.end();
                    }
                    catch (const NBError &error)
                    {
                        result = NBErrors::SetLast(error);
                    }
                    catch (const exception &e)
                    {
                        result = NBErrors::SetLast(e);
                    }
                    catch (...)
                    {
                        result = NBErrors::SetLast(NBErrors::NB_ERROR_EXTERNAL_CPP);
                    }
                    return result;
                }

                static NBResult NB_API TerminateNativeCallback(NBVoid * pContext)
                {
                    NBResult result = NBErrors::NB_OK;
                    try
                    {
                    }
                    catch (const NBError &error)
                    {
                        result = NBErrors::SetLast(error);
                    }
                    catch (const exception &e)
                    {
                        result = NBErrors::SetLast(e);
                    }
                    catch (...)
                    {
                        result = NBErrors::SetLast(NBErrors::NB_ERROR_EXTERNAL_CPP);
                    }
                    return result;
                }

                static NBResult NB_API NextTemplateNativeCallback(NBVoid * pContext, const NBVoid * * ppTemplateId, const NBVoid * * ppTemplate, NBSizeType * pstTemplateSize)
                {
                    NBResult result = NBErrors::NB_OK;
                    try
                    {
                        NBBiometricsTemplateIteratorEventHandler * pHandler = reinterpret_cast<NBBiometricsTemplateIteratorEventHandler*>(pContext);
                        *ppTemplateId = pHandler->m_iteratorBegin->first;
                        *ppTemplate = pHandler->m_iteratorBegin->second.m_pData.data();
                        *pstTemplateSize = pHandler->m_iteratorBegin->second.m_pData.size();
                        pHandler->m_iteratorBegin += 1;
                    }
                    catch (const NBError &error)
                    {
                        result = NBErrors::SetLast(error);
                    }
                    catch (const exception &e)
                    {
                        result = NBErrors::SetLast(e);
                    }
                    catch (...)
                    {
                        result = NBErrors::SetLast(NBErrors::NB_ERROR_EXTERNAL_CPP);
                    }
                    return result;
                }

                static NBResult NB_API HasNextTemplateNativeCallback(NBVoid * pContext, NBBool * pbValue)
                {
                    NBResult result = NBErrors::NB_OK;
                    try
                    {
                        NBBiometricsTemplateIteratorEventHandler * pHandler = reinterpret_cast<NBBiometricsTemplateIteratorEventHandler*>(pContext);
                        *pbValue = pHandler->m_iteratorBegin != pHandler->m_iteratorEnd ? NBTrue : NBFalse;
                    }
                    catch (const NBError &error)
                    {
                        result = NBErrors::SetLast(error);
                    }
                    catch (const exception &e)
                    {
                        result = NBErrors::SetLast(e);
                    }
                    catch (...)
                    {
                        result = NBErrors::SetLast(NBErrors::NB_ERROR_EXTERNAL_CPP);
                    }
                    return result;
                }
            };

            class NBBiometricsContextHandle
            {
                HNBBiometricsContext m_hHandle;
                bool m_bOwnsHandle;
            public:
                NBBiometricsContextHandle(HNBBiometricsContext hHandle, bool bOwnsHandle)
                    : m_hHandle(hHandle), m_bOwnsHandle(bOwnsHandle)
                {
                }

                ~NBBiometricsContextHandle()
                {
                    if (m_bOwnsHandle && m_hHandle) NBErrors::RequireNoErr(NBBiometricsContextDestroy(m_hHandle));
                }

                HNBDevice GetHandle() const
                {
                    return m_hHandle;
                }

                bool OwnsHandle() const
                {
                    return m_bOwnsHandle;
                }
            };

            shared_ptr<NBBiometricsContextHandle> m_hHandle;

            void CheckHandle() const
            {
                if (!m_hHandle || !m_hHandle->GetHandle()) NBThrowException(NBErrors::NB_ERROR_ARGUMENT_NULL, "m_hHandle is NULL");
            }

            static HNBBiometricsContext Create(const NBDevice& device, NBUInt uiFlags)
            {
                HNBBiometricsContext hContext = nullptr;
                NBErrors::RequireNoErr(NBBiometricsContextCreate(device.GetHandle(), uiFlags, &hContext));
                return hContext;
            }

            static HNBBiometricsContext Create(const NBDevice& device, const NBVoid * pLicense, NBSizeType stLicenseSize, NBUInt uiFlags)
            {
                HNBBiometricsContext hContext = nullptr;
                NBErrors::RequireNoErr(NBBiometricsContextCreateWithLicense(device.GetHandle(), pLicense, stLicenseSize, uiFlags, &hContext));
                return hContext;
            }

        public:
            NBBiometricsContext(HNBBiometricsContext hHandle, bool bOwnsHandle)
            {
                m_hHandle = make_shared<NBBiometricsContextHandle>(hHandle, bOwnsHandle);
            }

            NBBiometricsContext()
                : NBBiometricsContext(nullptr, false)
            {
            }

            NBBiometricsContext(HNBBiometricsContext hHandle)
                : NBBiometricsContext(hHandle, true)
            {
            }

            NBBiometricsContext(const NBDevice& device, NBUInt uiFlags)
                : NBBiometricsContext(Create(device, uiFlags), true)
            {
            }

            NBBiometricsContext(const NBDevice& device)
                : NBBiometricsContext(device, 0)
            {
            }

            NBBiometricsContext(const NBDevice& device, const NBVoid * pLicense, NBSizeType stLicenseSize, NBUInt uiFlags)
                : NBBiometricsContext(Create(device, pLicense, stLicenseSize, uiFlags), true)
            {
            }

            NBBiometricsContext(const NBDevice& device, const NBVoid * pLicense, NBSizeType stLicenseSize)
                : NBBiometricsContext(device, pLicense, stLicenseSize, 0)
            {
            }

            HNBBiometricsContext GetHandle() const
            {
                CheckHandle();
                return m_hHandle->GetHandle();
            }

            NBBiometricsAlgorithmInfo GetAlgorithmInfo() const
            {
                NBBiometricsAlgorithmInfo sAlgorithmInfo;
                NBErrors::RequireNoErr(NBBiometricsContextGetAlgorithmInfo(GetHandle(), &sAlgorithmInfo));
                return sAlgorithmInfo;
            }

            NBBool IsOperationRunning() const
            {
                NBBool value;
                NBErrors::RequireNoErr(NBBiometricsContextIsOperationRunning(GetHandle(), &value));
                return value;
            }

            void SetParameter(NBUInt uiParameter, NBInt iValue) const
            {
                NBErrors::RequireNoErr(NBBiometricsContextSetParameter(GetHandle(), uiParameter, iValue));
            }

            NBInt GetParameter(NBUInt uiParameter) const
            {
                NBInt value;
                NBErrors::RequireNoErr(NBBiometricsContextGetParameter(GetHandle(), uiParameter, &value));
                return value;
            }

            NBInt GetSecurityLevel(NBBiometricsSecurityLevel eSecurityLevel) const
            {
                NBInt value;
                NBErrors::RequireNoErr(NBBiometricsContextGetSecurityLevel(GetHandle(), eSecurityLevel, &value));
                return value;
            }

            NBBool IsTemplateTypeSupported(NBBiometricsTemplateType eTemplateType) const
            {
                NBBool value;
                NBErrors::RequireNoErr(NBBiometricsContextIsTemplateTypeSupported(GetHandle(), eTemplateType, &value));
                return value;
            }

            NBBiometricsTemplateTypeInfo GetTemplateTypeInfo(NBBiometricsTemplateType eTemplateType) const
            {
                NBBiometricsTemplateTypeInfo sTemplateTypeInfo;
                NBErrors::RequireNoErr(NBBiometricsContextGetTemplateTypeInfo(GetHandle(), eTemplateType, &sTemplateTypeInfo));
                return sTemplateTypeInfo;
            }

            vector<NBBiometricsTemplateTypeInfo> GetSupportedTemplateTypes() const
            {
                NBUInt uiTemplateTypesCount;
                NBErrors::RequireNoErr(NBBiometricsContextGetSupportedTemplateTypes(GetHandle(), nullptr, 0, &uiTemplateTypesCount));
                vector<NBBiometricsTemplateTypeInfo> arTemplateTypes(uiTemplateTypesCount);
                if (uiTemplateTypesCount > 0)
                {
                    NBUInt uiNewTemplateTypesCount;
                    vector<NBBiometricsTemplateType> areTemplateTypes(uiTemplateTypesCount);
                    NBErrors::RequireNoErr(NBBiometricsContextGetSupportedTemplateTypes(GetHandle(), areTemplateTypes.data(), NBUInt(areTemplateTypes.size()), &uiNewTemplateTypesCount));
                    for (NBUInt i = 0; i < min(uiTemplateTypesCount, uiNewTemplateTypesCount); i++)
                    {
                        arTemplateTypes[i] = GetTemplateTypeInfo(areTemplateTypes[i]);
                    }
                }
                return arTemplateTypes;
            }

            void CancelOperation() const
            {
                NBErrors::RequireNoErr(NBBiometricsContextCancelOperation(GetHandle()));
            }

            NBBiometricsTemplate ConvertTemplate(const NBBiometricsTemplate& sourceTemplate, NBBiometricsTemplateType eDestinationTemplateType) const
            {
                return ConvertTemplate(sourceTemplate, eDestinationTemplateType, 0);
            }

            NBBiometricsTemplate ConvertTemplate(const NBBiometricsTemplate& sourceTemplate, NBBiometricsTemplateType eDestinationTemplateType, NBUInt uiFlags) const
            {
                NBSizeType stTemplateSize;
                NBErrors::RequireNoErr(NBBiometricsContextGetMaxTemplateSize(GetHandle(), eDestinationTemplateType, &stTemplateSize));
                vector<NBByte> pTemplate(stTemplateSize);
                NBSizeType stActualSize;
                NBErrors::RequireNoErr(NBBiometricsContextConvertTemplate(GetHandle(), sourceTemplate.m_pData.data(), sourceTemplate.m_pData.size(), eDestinationTemplateType, uiFlags, pTemplate.data(), stTemplateSize, &stActualSize));
                return NBBiometricsTemplate(pTemplate.data(), stTemplateSize, stActualSize);
            }

            NBBiometricsTemplate LoadTemplate(NBBiometricsTemplateType eTemplateType, const NBByte * pData, NBSizeType stDataSize) const
            {
                return LoadTemplate(eTemplateType, pData, stDataSize, 0);
            }

            NBBiometricsTemplate LoadTemplate(NBBiometricsTemplateType eTemplateType, const NBByte * pData, NBSizeType stDataSize, NBUInt uiFlags) const
            {
                NBSizeType stTemplateSize;
                NBErrors::RequireNoErr(NBBiometricsContextGetMaxTemplateSize(GetHandle(), eTemplateType, &stTemplateSize));
                vector<NBByte> pTemplate(stTemplateSize);
                NBSizeType stActualSize;
                NBErrors::RequireNoErr(NBBiometricsContextLoadTemplateFromMemory(GetHandle(), eTemplateType, pData, stDataSize, uiFlags, pTemplate.data(), stTemplateSize, &stActualSize));
                return NBBiometricsTemplate(pTemplate.data(), stTemplateSize, stActualSize);
            }

            vector<NBByte> SaveTemplate(const NBBiometricsTemplate& biometricTemplate) const
            {
                return SaveTemplate(biometricTemplate, 0);
            }

            vector<NBByte> SaveTemplate(const NBBiometricsTemplate& biometricTemplate, NBUInt uiFlags) const
            {
                NBSizeType stDataSize;
                NBErrors::RequireNoErr(NBBiometricsContextGetMaxTemplateSize(GetHandle(), biometricTemplate.GetType(), &stDataSize));
                vector<NBByte> pData(stDataSize);
                NBSizeType stActualSize;
                NBErrors::RequireNoErr(NBBiometricsContextSaveTemplateToMemory(GetHandle(), biometricTemplate.m_pData.data(), biometricTemplate.m_pData.size(), uiFlags, pData.data(), stDataSize, &stActualSize));
                pData.resize(stActualSize);
                return pData;
            }

            NBBiometricsCreateEnrollTemplateResult CreateEnrollTemplate(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                NBDeviceScanFormat eScanFormat) const
            {
                return CreateEnrollTemplate(eTemplateType, eFingerPosition, eScanFormat, nullptr, nullptr);
            }

            NBBiometricsCreateEnrollTemplateResult CreateEnrollTemplate(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                const NBDeviceScanFormatInfo * psScanFormatInfo) const
            {
                return CreateEnrollTemplate(eTemplateType, eFingerPosition, psScanFormatInfo, nullptr, nullptr);
            }

            NBBiometricsCreateEnrollTemplateResult CreateEnrollTemplate(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                NBDeviceScanFormat eScanFormat, const NBBiometricsScanPreviewEvent pPreviewCallback, void * pParam) const
            {
                return CreateEnrollTemplate(eTemplateType, eFingerPosition, eScanFormat, NBDevice::SCAN_TIMEOUT_INFINITE, pPreviewCallback, pParam);
            }

            NBBiometricsCreateEnrollTemplateResult CreateEnrollTemplate(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                const NBDeviceScanFormatInfo * psScanFormatInfo, const NBBiometricsScanPreviewEvent pPreviewCallback, void * pParam) const
            {
                return CreateEnrollTemplate(eTemplateType, eFingerPosition, psScanFormatInfo, NBDevice::SCAN_TIMEOUT_INFINITE, pPreviewCallback, pParam);
            }

            NBBiometricsCreateEnrollTemplateResult CreateEnrollTemplate(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                NBDeviceScanFormat eScanFormat, NBInt iTimeout) const
            {
                return CreateEnrollTemplate(eTemplateType, eFingerPosition, eScanFormat, iTimeout, nullptr, nullptr);
            }

            NBBiometricsCreateEnrollTemplateResult CreateEnrollTemplate(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                const NBDeviceScanFormatInfo * psScanFormatInfo, NBInt iTimeout) const
            {
                return CreateEnrollTemplate(eTemplateType, eFingerPosition, psScanFormatInfo, iTimeout, nullptr, nullptr);
            }

            NBBiometricsCreateEnrollTemplateResult CreateEnrollTemplate(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                const NBDeviceScanFormatInfo * psScanFormatInfo, NBInt iTimeout, const NBBiometricsScanPreviewEvent pPreviewCallback, void * pParam) const
            {
                return CreateEnrollTemplate(eTemplateType, eFingerPosition, psScanFormatInfo, iTimeout, pPreviewCallback, pParam, 0);
            }

            NBBiometricsCreateEnrollTemplateResult CreateEnrollTemplate(NBBiometricsTemplateType eTemplateType,
                NBBiometricsFingerPosition eFingerPosition,
                NBDeviceScanFormat eScanFormat, NBInt iTimeout, const NBBiometricsScanPreviewEvent pPreviewCallback, void * pParam) const
            {
                return CreateEnrollTemplate(eTemplateType, eFingerPosition, eScanFormat, iTimeout, pPreviewCallback, pParam, 0);
            }

            NBBiometricsCreateEnrollTemplateResult CreateEnrollTemplate(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                const NBDeviceScanFormatInfo * psScanFormatInfo, NBInt iTimeout, const NBBiometricsScanPreviewEvent pPreviewCallback, void * pParam,
                NBUInt uiFlags) const
            {
                if (!psScanFormatInfo) NBThrowException(NBErrors::NB_ERROR_ARGUMENT_NULL, "psScanFormatInfo is NULL");
                return CreateEnrollTemplate(eTemplateType, eFingerPosition, psScanFormatInfo->eScanFormat, iTimeout, pPreviewCallback, pParam, uiFlags);
            }

            NBBiometricsCreateEnrollTemplateResult CreateEnrollTemplate(NBBiometricsTemplateType eTemplateType,
                NBBiometricsFingerPosition eFingerPosition,
                NBDeviceScanFormat eScanFormat, NBInt iTimeout, const NBBiometricsScanPreviewEvent pPreviewCallback, void * pParam,
                NBUInt uiFlags) const
            {
                NBSizeType stTemplateSize, stActualSize;
                unique_ptr<NBBiometricsScanPreviewEventHandler> pScanPreviewEvent(new NBBiometricsScanPreviewEventHandler(pPreviewCallback, pParam));
                NBBiometricsStatus eStatus;
                NBBiometricsScanParams sScanParams = { eScanFormat, iTimeout, pPreviewCallback ? NBBiometricsScanPreviewEventHandler::NativeCallback : nullptr, pScanPreviewEvent.get() };
                NBErrors::RequireNoErr(NBBiometricsContextGetMaxTemplateSize(GetHandle(), eTemplateType, &stTemplateSize));
                vector<NBByte> pTemplate(stTemplateSize);
                NBErrors::RequireNoErr(NBBiometricsContextCreateEnrollTemplateFromScan(GetHandle(), eTemplateType, eFingerPosition,
                    &sScanParams, uiFlags, pTemplate.data(), stTemplateSize, &eStatus, &stActualSize));
                if (eStatus == NBBiometricsStatusOk)
                {
                    return NBBiometricsCreateEnrollTemplateResult(eStatus, pTemplate.data(), stTemplateSize, stActualSize);
                }
                else
                {
                    return NBBiometricsCreateEnrollTemplateResult(eStatus);
                }
            }

            NBBiometricsExtractResult Extract(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                NBDeviceScanFormat eScanFormat) const
            {
                return Extract(eTemplateType, eFingerPosition, eScanFormat, nullptr, nullptr);
            }

            NBBiometricsExtractResult Extract(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                const NBDeviceScanFormatInfo * psScanFormatInfo) const
            {
                return Extract(eTemplateType, eFingerPosition, psScanFormatInfo, nullptr, nullptr);
            }

            NBBiometricsExtractResult Extract(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                NBDeviceScanFormat eScanFormat, const NBBiometricsScanPreviewEvent pPreviewCallback, void * pParam) const
            {
                return Extract(eTemplateType, eFingerPosition, eScanFormat, NBDevice::SCAN_TIMEOUT_INFINITE, pPreviewCallback, pParam);
            }

            NBBiometricsExtractResult Extract(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                const NBDeviceScanFormatInfo * psScanFormatInfo, const NBBiometricsScanPreviewEvent pPreviewCallback, void * pParam) const
            {
                return Extract(eTemplateType, eFingerPosition, psScanFormatInfo, NBDevice::SCAN_TIMEOUT_INFINITE, pPreviewCallback, pParam);
            }

            NBBiometricsExtractResult Extract(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                NBDeviceScanFormat eScanFormat, NBInt iTimeout) const
            {
                return Extract(eTemplateType, eFingerPosition, eScanFormat, iTimeout, nullptr, nullptr);
            }

            NBBiometricsExtractResult Extract(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                const NBDeviceScanFormatInfo * psScanFormatInfo, NBInt iTimeout) const
            {
                return Extract(eTemplateType, eFingerPosition, psScanFormatInfo, iTimeout, nullptr, nullptr);
            }

            NBBiometricsExtractResult Extract(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                const NBDeviceScanFormatInfo * psScanFormatInfo, NBInt iTimeout, const NBBiometricsScanPreviewEvent pPreviewCallback, void * pParam) const
            {
                return Extract(eTemplateType, eFingerPosition, psScanFormatInfo, iTimeout, pPreviewCallback, pParam, 0);
            }

            NBBiometricsExtractResult Extract(NBBiometricsTemplateType eTemplateType,
                NBBiometricsFingerPosition eFingerPosition,
                NBDeviceScanFormat eScanFormat, NBInt iTimeout, const NBBiometricsScanPreviewEvent pPreviewCallback, void * pParam) const
            {
                return Extract(eTemplateType, eFingerPosition, eScanFormat, iTimeout, pPreviewCallback, pParam, 0);
            }

            NBBiometricsExtractResult Extract(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                const NBDeviceScanFormatInfo * psScanFormatInfo, NBInt iTimeout, const NBBiometricsScanPreviewEvent pPreviewCallback, void * pParam,
                NBUInt uiFlags) const
            {
                if (!psScanFormatInfo) NBThrowException(NBErrors::NB_ERROR_ARGUMENT_NULL, "psScanFormatInfo is NULL");
                return Extract(eTemplateType, eFingerPosition, psScanFormatInfo->eScanFormat, iTimeout, pPreviewCallback, pParam, uiFlags);
            }

            NBBiometricsExtractResult Extract(NBBiometricsTemplateType eTemplateType,
                NBBiometricsFingerPosition eFingerPosition,
                NBDeviceScanFormat eScanFormat, NBInt iTimeout, const NBBiometricsScanPreviewEvent pPreviewCallback, void * pParam,
                NBUInt uiFlags) const
            {
                NBSizeType stTemplateSize, stActualSize;
                unique_ptr<NBBiometricsScanPreviewEventHandler> pScanPreviewEvent(new NBBiometricsScanPreviewEventHandler(pPreviewCallback, pParam));
                NBBiometricsStatus eStatus;
                NBBiometricsScanParams sScanParams = { eScanFormat, iTimeout, pPreviewCallback ? NBBiometricsScanPreviewEventHandler::NativeCallback : nullptr, pScanPreviewEvent.get() };
                NBErrors::RequireNoErr(NBBiometricsContextGetMaxTemplateSize(GetHandle(), eTemplateType, &stTemplateSize));
                vector<NBByte> pTemplate(stTemplateSize);
                NBErrors::RequireNoErr(NBBiometricsContextExtractFromScan(GetHandle(), eTemplateType, eFingerPosition,
                    &sScanParams, uiFlags, pTemplate.data(), stTemplateSize, &eStatus, &stActualSize));
                if (eStatus == NBBiometricsStatusOk)
                {
                    return NBBiometricsExtractResult(eStatus, pTemplate.data(), stTemplateSize, stActualSize);
                }
                else
                {
                    return NBBiometricsExtractResult(eStatus);
                }
            }

            NBBiometricsVerifyResult Verify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition, const NBDeviceScanFormatInfo& sScanFormatInfo,
                const NBBiometricsTemplate& templ, NBBiometricsSecurityLevel eSecurityLevel) const
            {
                return Verify(eTemplateType, eFingerPosition, sScanFormatInfo, templ, GetSecurityLevel(eSecurityLevel));
            }

            NBBiometricsVerifyResult Verify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition, const NBDeviceScanFormatInfo& sScanFormatInfo,
                const NBBiometricsTemplate& templ, NBInt iSecurityLevelValue) const
            {
                return Verify(eTemplateType, eFingerPosition, sScanFormatInfo, NBDevice::SCAN_TIMEOUT_INFINITE, templ, iSecurityLevelValue);
            }

            NBBiometricsVerifyResult Verify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition, NBDeviceScanFormat eScanFormat,
                const NBBiometricsTemplate& templ, NBBiometricsSecurityLevel eSecurityLevel) const
            {
                return Verify(eTemplateType, eFingerPosition, eScanFormat, templ, GetSecurityLevel(eSecurityLevel));
            }

            NBBiometricsVerifyResult Verify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition, NBDeviceScanFormat eScanFormat,
                const NBBiometricsTemplate& templ, NBInt iSecurityLevelValue) const
            {
                return Verify(eTemplateType, eFingerPosition, eScanFormat, NBDevice::SCAN_TIMEOUT_INFINITE, templ, iSecurityLevelValue);
            }

            NBBiometricsVerifyResult Verify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition, const NBDeviceScanFormatInfo& sScanFormatInfo, NBInt iScanTimeout,
                const NBBiometricsTemplate& templ, NBInt iSecurityLevelValue) const
            {
                return Verify(eTemplateType, eFingerPosition, sScanFormatInfo, iScanTimeout, nullptr, nullptr, templ, iSecurityLevelValue);
            }

            NBBiometricsVerifyResult Verify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition, const NBDeviceScanFormatInfo& sScanFormatInfo, NBInt iScanTimeout,
                const NBBiometricsTemplate& templ, NBBiometricsSecurityLevel eSecurityLevel) const
            {
                return Verify(eTemplateType, eFingerPosition, sScanFormatInfo, iScanTimeout, nullptr, nullptr, templ, eSecurityLevel);
            }

            NBBiometricsVerifyResult Verify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition, NBDeviceScanFormat eScanFormat, NBInt iScanTimeout,
                const NBBiometricsTemplate& templ, NBBiometricsSecurityLevel eSecurityLevel) const
            {
                return Verify(eTemplateType, eFingerPosition, eScanFormat, iScanTimeout, nullptr, nullptr, templ, eSecurityLevel);
            }

            NBBiometricsVerifyResult Verify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition, NBDeviceScanFormat eScanFormat, NBInt iScanTimeout,
                const NBBiometricsTemplate& templ, NBInt iSecurityLevelValue) const
            {
                return Verify(eTemplateType, eFingerPosition, eScanFormat, iScanTimeout, nullptr, nullptr, templ, iSecurityLevelValue);
            }

            NBBiometricsVerifyResult Verify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                const NBDeviceScanFormatInfo& sScanFormatInfo,
                const NBBiometricsScanPreviewEvent pPreviewCallback, void * pParam, const NBBiometricsTemplate& templ, NBBiometricsSecurityLevel eSecurityLevel) const
            {
                return Verify(eTemplateType, eFingerPosition, sScanFormatInfo.eScanFormat, pPreviewCallback, pParam, templ, eSecurityLevel);
            }

            NBBiometricsVerifyResult Verify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                const NBDeviceScanFormatInfo& sScanFormatInfo,
                const NBBiometricsScanPreviewEvent pPreviewCallback, void * pParam, const NBBiometricsTemplate& templ, NBInt iSecurityLevelValue) const
            {
                return Verify(eTemplateType, eFingerPosition, sScanFormatInfo.eScanFormat, pPreviewCallback, pParam, templ, iSecurityLevelValue);
            }

            NBBiometricsVerifyResult Verify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition, NBDeviceScanFormat eScanFormat,
                const NBBiometricsScanPreviewEvent pPreviewCallback, void * pParam, const NBBiometricsTemplate& templ,
                NBBiometricsSecurityLevel eSecurityLevel) const
            {
                return Verify(eTemplateType, eFingerPosition, eScanFormat, pPreviewCallback, pParam, templ, GetSecurityLevel(eSecurityLevel));
            }

            NBBiometricsVerifyResult Verify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition, NBDeviceScanFormat eScanFormat,
                const NBBiometricsScanPreviewEvent pPreviewCallback, void * pParam, const NBBiometricsTemplate& templ,
                NBInt iSecurityLevelValue) const
            {
                return Verify(eTemplateType, eFingerPosition, eScanFormat, NBDevice::SCAN_TIMEOUT_INFINITE, pPreviewCallback, pParam, templ,
                    iSecurityLevelValue);
            }

            NBBiometricsVerifyResult Verify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                NBDeviceScanFormat eScanFormat, NBInt iScanTimeout, const NBBiometricsScanPreviewEvent pPreviewCallback, void * pParam,
                const NBBiometricsTemplate& templ, NBBiometricsSecurityLevel eSecurityLevel) const
            {
                return Verify(eTemplateType, eFingerPosition, eScanFormat, iScanTimeout, pPreviewCallback, pParam, templ, GetSecurityLevel(eSecurityLevel));
            }

            NBBiometricsVerifyResult Verify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                NBDeviceScanFormat eScanFormat, NBInt iScanTimeout, const NBBiometricsScanPreviewEvent pPreviewCallback, void * pParam,
                const NBBiometricsTemplate& templ, NBBiometricsSecurityLevel eSecurityLevel, NBUInt uiFlags) const
            {
                return Verify(eTemplateType, eFingerPosition, eScanFormat, iScanTimeout, pPreviewCallback, pParam, templ, GetSecurityLevel(eSecurityLevel), uiFlags);
            }

            NBBiometricsVerifyResult Verify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                const NBDeviceScanFormatInfo& sScanFormatInfo, NBInt iScanTimeout, const NBBiometricsScanPreviewEvent pPreviewCallback, void * pParam,
                const NBBiometricsTemplate& templ, NBBiometricsSecurityLevel eSecurityLevel) const
            {
                return Verify(eTemplateType, eFingerPosition, sScanFormatInfo.eScanFormat, iScanTimeout, pPreviewCallback, pParam, templ, GetSecurityLevel(eSecurityLevel));
            }

            NBBiometricsVerifyResult Verify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                const NBDeviceScanFormatInfo& sScanFormatInfo, NBInt iScanTimeout, const NBBiometricsScanPreviewEvent pPreviewCallback, void * pParam,
                const NBBiometricsTemplate& templ, NBBiometricsSecurityLevel eSecurityLevel, NBUInt uiFlags) const
            {
                return Verify(eTemplateType, eFingerPosition, sScanFormatInfo.eScanFormat, iScanTimeout, pPreviewCallback, pParam, templ, GetSecurityLevel(eSecurityLevel), uiFlags);
            }

            NBBiometricsVerifyResult Verify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                const NBDeviceScanFormatInfo& sScanFormatInfo, NBInt iScanTimeout, const NBBiometricsScanPreviewEvent pPreviewCallback, void * pParam,
                const NBBiometricsTemplate& templ, NBInt iSecurityLevelValue)  const
            {
                return Verify(eTemplateType, eFingerPosition, sScanFormatInfo.eScanFormat, iScanTimeout, pPreviewCallback, pParam, templ, iSecurityLevelValue, 0);
            }

            NBBiometricsVerifyResult Verify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                const NBDeviceScanFormatInfo& sScanFormatInfo, NBInt iScanTimeout, const NBBiometricsScanPreviewEvent pPreviewCallback, void * pParam,
                const NBBiometricsTemplate& templ, NBInt iSecurityLevelValue, NBUInt uiFlags)  const
            {
                return Verify(eTemplateType, eFingerPosition, sScanFormatInfo.eScanFormat, iScanTimeout, pPreviewCallback, pParam, templ, iSecurityLevelValue, uiFlags);
            }

            NBBiometricsVerifyResult Verify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                NBDeviceScanFormat eScanFormat, NBInt iScanTimeout, const NBBiometricsScanPreviewEvent pPreviewCallback, void * pParam,
                const NBBiometricsTemplate& templ, NBInt iSecurityLevelValue) const
            {
                return Verify(eTemplateType, eFingerPosition, eScanFormat, iScanTimeout, pPreviewCallback, pParam, templ, iSecurityLevelValue, 0);
            }

            NBBiometricsVerifyResult Verify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                NBDeviceScanFormat eScanFormat, NBInt iScanTimeout, const NBBiometricsScanPreviewEvent pPreviewCallback, void * pParam,
                const NBBiometricsTemplate& templ, NBInt iSecurityLevelValue, NBUInt uiFlags) const
            {
                unique_ptr<NBBiometricsScanPreviewEventHandler> pScanPreviewEvent(new NBBiometricsScanPreviewEventHandler(pPreviewCallback, pParam));
                NBBiometricsStatus eStatus;
                NBBiometricsScanParams sScanParams = { eScanFormat, iScanTimeout, pPreviewCallback ? NBBiometricsScanPreviewEventHandler::NativeCallback : nullptr, pScanPreviewEvent.get() };
                NBBiometricsVerifyResultDetails sResultDetails;
                NBErrors::RequireNoErr(NBBiometricsContextVerifyFromScan(GetHandle(), eTemplateType, eFingerPosition,
                    &sScanParams, templ.m_pData.data(), templ.m_pData.size(), iSecurityLevelValue, uiFlags, &eStatus, &sResultDetails));
                return NBBiometricsVerifyResult(eStatus, sResultDetails);
            }

            NBBiometricsVerifyResult Verify(const NBBiometricsTemplate& templ, const NBBiometricsTemplate& otherTempl,
                NBBiometricsSecurityLevel eSecurityLevel) const
            {
                return Verify(templ, otherTempl, GetSecurityLevel(eSecurityLevel));
            }

            NBBiometricsVerifyResult Verify(const NBBiometricsTemplate& templ, const NBBiometricsTemplate& otherTempl,
                NBInt iSecurityLevelValue) const
            {
                return Verify(templ, otherTempl, iSecurityLevelValue, 0);
            }

            NBBiometricsVerifyResult Verify(const NBBiometricsTemplate& templ, const NBBiometricsTemplate& otherTempl,
                NBBiometricsSecurityLevel eSecurityLevel, NBUInt uiFlags) const
            {
                return Verify(templ, otherTempl, GetSecurityLevel(eSecurityLevel), uiFlags);
            }

            NBBiometricsVerifyResult Verify(const NBBiometricsTemplate& templ, const NBBiometricsTemplate& otherTempl,
                NBInt iSecurityLevelValue, NBUInt uiFlags) const
            {
                NBBiometricsStatus eStatus;
                NBBiometricsVerifyResultDetails sVerifyResultDetails;
                NBErrors::RequireNoErr(NBBiometricsContextVerifyFromTemplate(GetHandle(),
                    templ.m_pData.data(), templ.m_pData.size(),
                    otherTempl.m_pData.data(), otherTempl.m_pData.size(),
                    iSecurityLevelValue, uiFlags, &eStatus, &sVerifyResultDetails));
                return NBBiometricsVerifyResult(eStatus, sVerifyResultDetails);
            }

            template<typename T>
            NBBiometricsIdentifyResult Identify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                NBDeviceScanFormat eScanFormat, T otherTemplates, NBInt iSecurityLevelValue) const
            {
                return Identify(eTemplateType, eFingerPosition, eScanFormat, NBDevice::SCAN_TIMEOUT_INFINITE, otherTemplates, iSecurityLevelValue);
            }

            template<typename T>
            NBBiometricsIdentifyResult Identify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                const NBDeviceScanFormatInfo& sScanFormatInfo, T otherTemplates, NBInt iSecurityLevelValue) const
            {
                return Identify(eTemplateType, eFingerPosition, sScanFormatInfo.eScanFormat, otherTemplates, iSecurityLevelValue);
            }

            template<typename T>
            NBBiometricsIdentifyResult Identify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                NBDeviceScanFormat eScanFormat, T otherTemplates,
                NBBiometricsSecurityLevel eSecurityLevel) const
            {
                return Identify(eTemplateType, eFingerPosition, eScanFormat, otherTemplates, GetSecurityLevel(eSecurityLevel));
            }

            template<typename T>
            NBBiometricsIdentifyResult Identify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                const NBDeviceScanFormatInfo& sScanFormatInfo, T otherTemplates,
                NBBiometricsSecurityLevel eSecurityLevel) const
            {
                return Identify(eTemplateType, eFingerPosition, sScanFormatInfo.eScanFormat, otherTemplates, eSecurityLevel);
            }

            template<typename T>
            NBBiometricsIdentifyResult Identify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                NBDeviceScanFormat eScanFormat, const NBBiometricsScanPreviewEvent pPreviewCallback, void * pParam,
                T otherTemplates, NBInt iSecurityLevelValue) const
            {
                return Identify(eTemplateType, eFingerPosition, eScanFormat, NBDevice::SCAN_TIMEOUT_INFINITE, pPreviewCallback, pParam, otherTemplates, iSecurityLevelValue);
            }

            template<typename T>
            NBBiometricsIdentifyResult Identify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                const NBDeviceScanFormatInfo& sScanFormatInfo, const NBBiometricsScanPreviewEvent pPreviewCallback, void * pParam,
                T otherTemplates, NBInt iSecurityLevelValue) const
            {
                return Identify(eTemplateType, eFingerPosition, sScanFormatInfo.eScanFormat, pPreviewCallback, pParam, otherTemplates, iSecurityLevelValue);
            }

            template<typename T>
            NBBiometricsIdentifyResult Identify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                NBDeviceScanFormat eScanFormat, const NBBiometricsScanPreviewEvent pPreviewCallback, void * pParam,
                T otherTemplates, NBBiometricsSecurityLevel eSecurityLevel) const
            {
                return Identify(eTemplateType, eFingerPosition, eScanFormat, pPreviewCallback, pParam, otherTemplates, GetSecurityLevel(eSecurityLevel));
            }

            template<typename T>
            NBBiometricsIdentifyResult Identify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                const NBDeviceScanFormatInfo& sScanFormatInfo, const NBBiometricsScanPreviewEvent pPreviewCallback, void * pParam,
                T otherTemplates, NBBiometricsSecurityLevel eSecurityLevel) const
            {
                return Identify(eTemplateType, eFingerPosition, sScanFormatInfo.eScanFormat, pPreviewCallback, pParam, otherTemplates, eSecurityLevel);
            }

            template<typename T>
            NBBiometricsIdentifyResult Identify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                NBDeviceScanFormat eScanFormat, NBInt iScanTimeout,
                T otherTemplates, NBInt iSecurityLevelValue) const
            {
                return Identify(eTemplateType, eFingerPosition, eScanFormat, iScanTimeout, NULL, NULL, otherTemplates, iSecurityLevelValue);
            }

            template<typename T>
            NBBiometricsIdentifyResult Identify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                const NBDeviceScanFormatInfo& sScanFormatInfo, NBInt iScanTimeout,
                T otherTemplates, NBInt iSecurityLevelValue) const
            {
                return Identify(eTemplateType, eFingerPosition, sScanFormatInfo.eScanFormat, iScanTimeout, otherTemplates, iSecurityLevelValue);
            }

            template<typename T>
            NBBiometricsIdentifyResult Identify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                NBDeviceScanFormat eScanFormat, NBInt iScanTimeout,
                T otherTemplates, NBBiometricsSecurityLevel eSecurityLevel) const
            {
                return Identify(eTemplateType, eFingerPosition, eScanFormat, iScanTimeout, otherTemplates, GetSecurityLevel(eSecurityLevel));
            }

            template<typename T>
            NBBiometricsIdentifyResult Identify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                const NBDeviceScanFormatInfo& sScanFormatInfo, NBInt iScanTimeout, T otherTemplates,
                NBBiometricsSecurityLevel eSecurityLevel) const
            {
                return Identify(eTemplateType, eFingerPosition, sScanFormatInfo.eScanFormat, iScanTimeout, otherTemplates, eSecurityLevel);
            }

            template<typename T>
            NBBiometricsIdentifyResult Identify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                const NBDeviceScanFormatInfo& sScanFormatInfo, NBInt iScanTimeout, const NBBiometricsScanPreviewEvent pPreviewCallback, void * pParam,
                T otherTemplates, NBBiometricsSecurityLevel eSecurityLevel) const
            {
                return Identify(eTemplateType, eFingerPosition, sScanFormatInfo.eScanFormat, iScanTimeout, pPreviewCallback, pParam, otherTemplates,
                    GetSecurityLevel(eSecurityLevel));
            }

            template<typename T>
            NBBiometricsIdentifyResult Identify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                NBDeviceScanFormat eScanFormat, NBInt iScanTimeout, const NBBiometricsScanPreviewEvent pPreviewCallback, void * pParam,
                T otherTemplates, NBBiometricsSecurityLevel eSecurityLevel) const
            {
                return Identify(eTemplateType, eFingerPosition, eScanFormat, iScanTimeout, pPreviewCallback, pParam, otherTemplates,
                    GetSecurityLevel(eSecurityLevel));
            }

            template<typename T>
            NBBiometricsIdentifyResult Identify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                const NBDeviceScanFormatInfo& sScanFormatInfo, NBInt iScanTimeout, const NBBiometricsScanPreviewEvent pPreviewCallback, void * pParam,
                T otherTemplates, NBInt iSecurityLevelValue) const
            {
                return Identify(eTemplateType, eFingerPosition, sScanFormatInfo.eScanFormat, iScanTimeout, pPreviewCallback, pParam, otherTemplates,
                    iSecurityLevelValue);
            }

            template<typename T>
            NBBiometricsIdentifyResult Identify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                NBDeviceScanFormat eScanFormat, NBInt iScanTimeout, const NBBiometricsScanPreviewEvent pPreviewCallback, void * pParam,
                T otherTemplates, NBInt iSecurityLevelValue) const
            {
                return Identify(eTemplateType, eFingerPosition, eScanFormat, iScanTimeout, pPreviewCallback, pParam, otherTemplates, iSecurityLevelValue, 0);
            }

            template<typename T>
            NBBiometricsIdentifyResult Identify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                const NBDeviceScanFormatInfo& sScanFormatInfo, NBInt iScanTimeout, const NBBiometricsScanPreviewEvent pPreviewCallback, void * pParam,
                T otherTemplates, NBBiometricsSecurityLevel eSecurityLevel, NBUInt uiFlags) const
            {
                return Identify(eTemplateType, eFingerPosition, sScanFormatInfo.eScanFormat, iScanTimeout, pPreviewCallback, pParam, otherTemplates,
                    eSecurityLevel, uiFlags);
            }

            template<typename T>
            NBBiometricsIdentifyResult Identify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                NBDeviceScanFormat eScanFormat, NBInt iScanTimeout, const NBBiometricsScanPreviewEvent pPreviewCallback, void * pParam,
                T otherTemplates, NBBiometricsSecurityLevel eSecurityLevel, NBUInt uiFlags) const
            {
                return Identify(eTemplateType, eFingerPosition, eScanFormat, iScanTimeout, pPreviewCallback, pParam, otherTemplates,
                    GetSecurityLevel(eSecurityLevel), uiFlags);
            }

            template<typename T>
            NBBiometricsIdentifyResult Identify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                const NBDeviceScanFormatInfo& sScanFormatInfo, NBInt iScanTimeout, const NBBiometricsScanPreviewEvent pPreviewCallback, void * pParam,
                T otherTemplates, NBInt iSecurityLevelValue, NBUInt uiFlags) const
            {
                return Identify(eTemplateType, eFingerPosition, sScanFormatInfo.eScanFormat, iScanTimeout, pPreviewCallback, pParam, otherTemplates,
                    iSecurityLevelValue, uiFlags);
            }

            template<typename T>
            NBBiometricsIdentifyResult Identify(NBBiometricsTemplateType eTemplateType, NBBiometricsFingerPosition eFingerPosition,
                NBDeviceScanFormat eScanFormat, NBInt iScanTimeout, const NBBiometricsScanPreviewEvent pPreviewCallback, void * pParam,
                T otherTemplates, NBInt iSecurityLevelValue, NBUInt uiFlags) const
            {
                unique_ptr<NBBiometricsScanPreviewEventHandler> pScanPreviewEvent(new NBBiometricsScanPreviewEventHandler(pPreviewCallback, pParam));
                unique_ptr<NBBiometricsTemplateIteratorEventHandler<T>> pIteratorEvent(new NBBiometricsTemplateIteratorEventHandler<T>(otherTemplates));
                NBBiometricsStatus eStatus;
                NBBiometricsScanParams sScanParams = { eScanFormat, iScanTimeout, pPreviewCallback ? NBBiometricsScanPreviewEventHandler::NativeCallback : nullptr, pScanPreviewEvent.get() };
                NBBiometricsTemplateIterator sIterator = { pIteratorEvent.get(), NBBiometricsTemplateIteratorEventHandler<T>::InitNativeCallback,
                    NBBiometricsTemplateIteratorEventHandler<T>::TerminateNativeCallback, NBBiometricsTemplateIteratorEventHandler<T>::NextTemplateNativeCallback,
                    NBBiometricsTemplateIteratorEventHandler<T>::HasNextTemplateNativeCallback, nullptr };
                NBBiometricsIdentifyResultDetails sResultDetails;
                NBErrors::RequireNoErr(NBBiometricsContextIdentifyFromScan(GetHandle(), eTemplateType, eFingerPosition,
                    &sScanParams, &sIterator, iSecurityLevelValue, uiFlags, &eStatus, &sResultDetails));
                return NBBiometricsIdentifyResult(eStatus, sResultDetails);
            }

            template<typename T>
            NBBiometricsIdentifyResult Identify(const NBBiometricsTemplate& templ, T otherTemplates,
                NBBiometricsSecurityLevel eSecurityLevel) const
            {
                return Identify(templ, otherTemplates, GetSecurityLevel(eSecurityLevel));
            }

            template<typename T>
            NBBiometricsIdentifyResult Identify(const NBBiometricsTemplate& templ,
                T otherTemplates, NBInt iSecurityLevelValue) const
            {
                return Identify(templ, otherTemplates, iSecurityLevelValue, 0);
            }

            template<typename T>
            NBBiometricsIdentifyResult Identify(const NBBiometricsTemplate& templ,
                T otherTemplates, NBBiometricsSecurityLevel eSecurityLevel, NBUInt uiFlags) const
            {
                return Identify(templ, otherTemplates, GetSecurityLevel(eSecurityLevel), uiFlags);
            }

            template<typename T>
            NBBiometricsIdentifyResult Identify(const NBBiometricsTemplate& templ,
                T otherTemplates, NBInt iSecurityLevelValue, NBUInt uiFlags) const
            {
                unique_ptr<NBBiometricsTemplateIteratorEventHandler<T>> pIteratorEvent(new NBBiometricsTemplateIteratorEventHandler<T>(otherTemplates));
                NBBiometricsStatus eStatus;
                NBBiometricsTemplateIterator sIterator = { pIteratorEvent.get(), NBBiometricsTemplateIteratorEventHandler<T>::InitNativeCallback,
                    NBBiometricsTemplateIteratorEventHandler<T>::TerminateNativeCallback, NBBiometricsTemplateIteratorEventHandler<T>::NextTemplateNativeCallback,
                    NBBiometricsTemplateIteratorEventHandler<T>::HasNextTemplateNativeCallback, nullptr };
                NBBiometricsIdentifyResultDetails sResultDetails;
                NBErrors::RequireNoErr(NBBiometricsContextIdentifyFromTemplate(GetHandle(), templ.m_pData.data(), templ.m_pData.size(),
                    &sIterator, iSecurityLevelValue, uiFlags, &eStatus, &sResultDetails));
                return NBBiometricsIdentifyResult(eStatus, sResultDetails);
            }
        };

        class NBBiometricsScanPreviewEventArgs : public EventArgs
        {
            friend class NBBiometricsContext;

            NBBiometricsContext m_context;
            NBBiometricsStatus m_eBiometricsStatus;
            NBDevice m_device;
            NBDeviceScanStatus m_eScanStatus;
            NBDeviceScanFormatInfo m_sScanFormatInfo;
            const NBByte * m_pImage;
            NBSizeType m_stImageSize;
            NBDeviceScanPreviewDetails m_sScanPreviewDetails;

            NBBiometricsScanPreviewEventArgs(const NBBiometricsContext& context, NBBiometricsStatus eBiometricsStatus, const NBDevice& device, NBDeviceScanStatus eScanStatus, const NBDeviceScanFormatInfo * psScanFormatInfo, const NBByte * pImage, NBSizeType stImageSize, const NBDeviceScanPreviewDetails * psScanPreviewDetails, void * pParam)
                : EventArgs(pParam), m_context(context), m_eBiometricsStatus(eBiometricsStatus), m_device(device), m_eScanStatus(eScanStatus), m_pImage(pImage), m_stImageSize(stImageSize)
            {
                if (psScanFormatInfo) m_sScanFormatInfo = *psScanFormatInfo;
                if (psScanPreviewDetails) m_sScanPreviewDetails = *psScanPreviewDetails;
            }
        public:
            const NBBiometricsContext& GetBiometricsContext() const
            {
                return m_context;
            }

            NBBiometricsStatus GetBiometricsStatus() const
            {
                return m_eBiometricsStatus;
            }

            const NBDevice& GetDevice() const
            {
                return m_device;
            }

            NBDeviceScanStatus GetScanStatus() const
            {
                return m_eScanStatus;
            }

            void SetScanStatus(NBDeviceScanStatus eStatus)
            {
                m_eScanStatus = eStatus;
            }

            const NBDeviceScanFormatInfo& GetFormat() const
            {
                return m_sScanFormatInfo;
            }

            const NBByte * GetImage() const
            {
                return m_pImage;
            }

            NBSizeType GetImageSize() const
            {
                return m_stImageSize;
            }

            NBInt GetFingerDetectValue() const
            {
                return m_sScanPreviewDetails.iFingerDetectValue;
            }
        };

        inline NBResult NB_API NBBiometricsContext::NBBiometricsScanPreviewEventHandler::NativeCallback(const HNBBiometricsContext hContext, NBBiometricsStatus eBiometricsStatus, const HNBDevice hDevice, NBDeviceScanStatus * peStatus, const NBDeviceScanFormatInfo * psScanFormatInfo, const NBByte * pBuffer, NBSizeType stBufferSize, const NBDeviceScanPreviewDetails * psDetails, void * pParam)
        {
            NBResult result = NBErrors::NB_OK;
            try
            {
                NBBiometricsScanPreviewEventHandler * pHandler = reinterpret_cast<NBBiometricsScanPreviewEventHandler*>(pParam);
                NBBiometricsScanPreviewEventArgs args(NBBiometricsContext(hContext, false), eBiometricsStatus, NBDevice(hDevice, false), *peStatus, psScanFormatInfo, pBuffer, stBufferSize, psDetails, pHandler->m_pParam);
                pHandler->m_callback(args);
                *peStatus = args.GetScanStatus();
            }
            catch (const NBError &error)
            {
                result = NBErrors::SetLast(error);
            }
            catch (const exception &e)
            {
                result = NBErrors::SetLast(e);
            }
            catch (...)
            {
                result = NBErrors::SetLast(NBErrors::NB_ERROR_EXTERNAL_CPP);
            }
            return result;
        }
    }
}

#endif // !NB_BIOMETRICS_CONTEXT_HPP
