#ifndef NB_ERRORS_HPP
#define NB_ERRORS_HPP

#include <NBTypes.hpp>
#include <exception>
#include <stdexcept>
#include <vector>
#include <ios>
#include <typeinfo>

namespace NextBiometrics
{
    namespace System
    {
        #include <NBErrors.h>

        #undef NB_OK
        #undef NB_ERROR_FAILED
        #undef NB_ERROR_ARGUMENT
        #undef NB_ERROR_ARGUMENT_NULL
        #undef NB_ERROR_ARGUMENT_OUT_OF_RANGE
        #undef NB_ERROR_INVALID_ENUM_ARGUMENT
        #undef NB_ERROR_INSUFFICIENT_BUFFER
        #undef NB_ERROR_FORMAT
        #undef NB_ERROR_INDEX_OUT_OF_RANGE
        #undef NB_ERROR_MEMORY
        #undef NB_ERROR_OUT_OF_MEMORY
        #undef NB_ERROR_MEMORY_CORRUPTION
        #undef NB_ERROR_ARITHMETIC
        #undef NB_ERROR_ARITHMETIC_DIVIDE_BY_ZERO
        #undef NB_ERROR_ARITHMETIC_MATRIX_NOT_SQUARE
        #undef NB_ERROR_OVERFLOW
        #undef NB_ERROR_OPERATION
        #undef NB_ERROR_NOT_SUPPORTED
        #undef NB_ERROR_NOT_IMPLEMENTED
        #undef NB_ERROR_INVALID_OPERATION
        #undef NB_ERROR_OPERATION_CANCELED
        #undef NB_ERROR_TIMEOUT
        #undef NB_ERROR_IO
        #undef NB_ERROR_IO_DEVICE_BUSY
        #undef NB_ERROR_IO_DEVICE_NOT_ACTIVE
        #undef NB_ERROR_IO_DEVICE_SENSOR_FAILED
        #undef NB_ERROR_IO_DEVICE_NOT_CALIBRATED
        #undef NB_ERROR_IO_DEVICE_AUTHENTICATION_FAILED
        #undef NB_ERROR_IO_NO_DEVICES
        #undef NB_ERROR_IO_COMMAND_FAILED
        #undef NB_ERROR_IO_COMMAND_RESPONSE_FAILED
        #undef NB_ERROR_IO_COMMUNICATION_FAILED
        #undef NB_ERROR_IO_DATA_FIELD_INVALID
        #undef NB_ERROR_IO_DATA_LENGTH_INVALID
        #undef NB_ERROR_IO_PARAMETER_FIELD_INVALID
        #undef NB_ERROR_IO_DCA
        #undef NB_ERROR_IO_MCU
        #undef NB_ERROR_IO_OPERATION_CONDITIONS_INVALID
        #undef NB_ERROR_IO_UNKNOWN_COMMAND
        #undef NB_ERROR_IO_FLASH
        #undef NB_ERROR_IO_SOCKET
        #undef NB_ERROR_EXTERNAL
        #undef NB_ERROR_EXTERNAL_WIN32
        #undef NB_ERROR_EXTERNAL_SYS
        #undef NB_ERROR_EXTERNAL_CPP
        #undef NB_ERROR_EXTERNAL_DOTNET
        #undef NB_ERROR_EXTERNAL_JVM
        #undef NB_ERROR_EXTERNAL_COM

        #undef NBFailed
        #undef NBSucceeded

        class NBError
        {
            NBResult m_code;
            NBString m_message;
        public:
            NBError(NBResult code, const NBString &message)
                : m_code(code), m_message(message)
            {
            }

            NBResult GetCode() const
            {
                return m_code;
            }

            const NBString& GetMessage() const
            {
                return m_message;
            }
        };

        inline void NB_NO_RETURN NBThrowException(NBResult code, const NBString &message) { throw NBError(code, message); }

        class NBErrors
        {
            static void RaiseError(NBResult result)
            {
                if (NBSucceeded(result)) NBThrowException(NB_ERROR_ARGUMENT_OUT_OF_RANGE, "error is succeeded");
                throw NBError(result, GetMessage(result));
            }

        public:
            static const NBResult NB_OK = 0;
            static const NBResult NB_ERROR_FAILED = -100;
            static const NBResult NB_ERROR_ARGUMENT = -200;
                static const NBResult NB_ERROR_ARGUMENT_NULL = -201;
                static const NBResult NB_ERROR_ARGUMENT_OUT_OF_RANGE = -202;
                static const NBResult NB_ERROR_INVALID_ENUM_ARGUMENT = -203;
                static const NBResult NB_ERROR_INSUFFICIENT_BUFFER = -204;
                static const NBResult NB_ERROR_INDEX_OUT_OF_RANGE = -205;
            static const NBResult NB_ERROR_FORMAT = -300;
            static const NBResult NB_ERROR_MEMORY = -400;
                static const NBResult NB_ERROR_OUT_OF_MEMORY = -401;
                static const NBResult NB_ERROR_MEMORY_CORRUPTION = -402;
            static const NBResult NB_ERROR_ARITHMETIC = -500;
                static const NBResult NB_ERROR_ARITHMETIC_DIVIDE_BY_ZERO = -501;
                static const NBResult NB_ERROR_ARITHMETIC_MATRIX_NOT_SQUARE = -502;
                static const NBResult NB_ERROR_OVERFLOW = -503;
            static const NBResult NB_ERROR_OPERATION = -600;
                static const NBResult NB_ERROR_NOT_SUPPORTED = -601;
                static const NBResult NB_ERROR_NOT_IMPLEMENTED = -602;
                static const NBResult NB_ERROR_INVALID_OPERATION = -603;
                static const NBResult NB_ERROR_OPERATION_CANCELED = -604;
                static const NBResult NB_ERROR_TIMEOUT = -605;
            static const NBResult NB_ERROR_IO = -700;
                static const NBResult NB_ERROR_IO_DEVICE_BUSY = -701;
                static const NBResult NB_ERROR_IO_DEVICE_NOT_ACTIVE = -702;
                static const NBResult NB_ERROR_IO_DEVICE_SENSOR_FAILED = -703;
                static const NBResult NB_ERROR_IO_DEVICE_NOT_CALIBRATED = -704;
                static const NBResult NB_ERROR_IO_DEVICE_AUTHENTICATION_FAILED = -705;
                static const NBResult NB_ERROR_IO_NO_DEVICES = -706;
                static const NBResult NB_ERROR_IO_COMMAND_FAILED = -707;
                static const NBResult NB_ERROR_IO_COMMAND_RESPONSE_FAILED = -708;
                static const NBResult NB_ERROR_IO_COMMUNICATION_FAILED = -709;
                static const NBResult NB_ERROR_IO_DATA_FIELD_INVALID = -710;
                static const NBResult NB_ERROR_IO_DATA_LENGTH_INVALID = -711;
                static const NBResult NB_ERROR_IO_PARAMETER_FIELD_INVALID = -712;
                static const NBResult NB_ERROR_IO_DCA = -713;
                static const NBResult NB_ERROR_IO_MCU = -714;
                static const NBResult NB_ERROR_IO_OPERATION_CONDITIONS_INVALID = -715;
                static const NBResult NB_ERROR_IO_UNKNOWN_COMMAND = -716;
                static const NBResult NB_ERROR_IO_FLASH = -717;
                static const NBResult NB_ERROR_IO_SOCKET = -718;
            static const NBResult NB_ERROR_EXTERNAL = -800;
                static const NBResult NB_ERROR_EXTERNAL_WIN32 = -801;
                static const NBResult NB_ERROR_EXTERNAL_SYS = -802;
                static const NBResult NB_ERROR_EXTERNAL_CPP = -803;
                static const NBResult NB_ERROR_EXTERNAL_DOTNET = -804;
                static const NBResult NB_ERROR_EXTERNAL_JVM = -805;
                static const NBResult NB_ERROR_EXTERNAL_COM = -806;

            static NBResult RequireNoErr(NBResult result)
            {
                if (NBFailed(result))
                {
                    RaiseError(result);
                }
                return result;
            }

			template<typename T>
			static T* RequireNoNull(T *value)
			{
				if (value == NULL)
				{
					RaiseError(NBErrors::NB_ERROR_ARGUMENT_NULL);
				}
				return value;
			}

            static bool NBFailed(NBResult result)
            {
                return result != NB_OK;
            }

            static bool NBSucceeded(NBResult result)
            {
                return !NBFailed(result);
            }

            static NBString GetMessage(NBResult result)
            {
                NBUInt uiValueLength;
                if (NBSucceeded(NBErrorsGetMessageA(result, NULL, 0, &uiValueLength)))
                {
                    std::vector<NBAChar> szMessage(uiValueLength + 1);
                    if (NBSucceeded(NBErrorsGetMessageA(result, &szMessage[0], (NBUInt)szMessage.size(), &uiValueLength)))
                    {
                        return ToNBString(&szMessage[0], uiValueLength);
                    }
                }
                return NULL;
            }

            static NB_NO_INLINE NBResult SetLast(const NBResult error)
            {
                NBErrorsSetLastA(error, FromNBString(GetMessage(error)), 0, NULL, 0x01000000);
                return error;
            }

            static NB_NO_INLINE NBResult SetLast(const NBError &error)
            {
                NBErrorsSetLastA(error.GetCode(), FromNBString(error.GetMessage()), 0, NULL, 0x01000000);
                return error.GetCode();
            }

            static NB_NO_INLINE NBResult SetLast(const std::exception &error)
            {
                NBResult code;
                const std::type_info &errorType = typeid(error);
                if (typeid(std::exception) == errorType)
                {
                    code = NB_ERROR_FAILED;
                }
                else if (dynamic_cast<const std::bad_alloc *>(&error))
                {
                    code = NB_ERROR_OUT_OF_MEMORY;
                }
                else if (dynamic_cast<const std::bad_cast *>(&error))
                {
                    code = NB_ERROR_MEMORY_CORRUPTION;
                }
                else if (dynamic_cast<const std::runtime_error *>(&error))
                {
                    if (typeid(std::runtime_error) == errorType)
                    {
                        code = NB_ERROR_FAILED;
                    }
                    else if (dynamic_cast<const std::overflow_error *>(&error))
                    {
                        code = NB_ERROR_OVERFLOW;
                    }
                    else if (dynamic_cast<const std::range_error *>(&error) || dynamic_cast<const std::underflow_error *>(&error))
                    {
                        code = NB_ERROR_ARITHMETIC;
                    }
                    else if (dynamic_cast<const std::ios_base::failure *>(&error))
                    {
                        code = NB_ERROR_IO;
                    }
                    else
                    {
                        code = NB_ERROR_EXTERNAL_CPP;
                    }
                }
                else if (dynamic_cast<const std::logic_error *>(&error))
                {
                    if (typeid(std::logic_error) == errorType)
                    {
                        code = NB_ERROR_FAILED;
                    }
                    else if (dynamic_cast<const std::invalid_argument *>(&error))
                    {
                        code = NB_ERROR_ARGUMENT;
                    }
                    else if (dynamic_cast<const std::out_of_range *>(&error))
                    {
                        code = NB_ERROR_ARGUMENT_OUT_OF_RANGE;
                    }
                    else if (dynamic_cast<const std::domain_error *>(&error))
                    {
                        code = NB_ERROR_ARGUMENT;
                    }
                    else
                    {
                        code = NB_ERROR_EXTERNAL_CPP;
                    }
                }
                else
                {
                    code = NB_ERROR_EXTERNAL_CPP;
                }
                NBErrorsSetLastA(code, FromNBString(error.what()), 0, NULL, 0x01000000);
                return code;
            }
        };

        inline void NB_NO_RETURN NBThrowException(NBResult code) { throw NBError(code, NBErrors::GetMessage(code)); }
    }
}

#endif // !NB_ERRORS_HPP
