#ifndef NB_TYPES_HPP
#define NB_TYPES_HPP

#if !defined(NB_FRAMEWORK_NO_AUTO_DETECT)
    #if defined(_MFC_VER)
        #define NB_FRAMEWORK_MFC
    #elif defined(wxMAJOR_VERSION)
        #define NB_FRAMEWORK_WX
    #elif defined(QT_VERSION) || defined(QT_CORE_LIB)
        #define NB_FRAMEWORK_QT
    #endif
#endif

#if defined(NB_FRAMEWORK_MFC)
    #include <afx.h>
#elif defined (NB_FRAMEWORK_WX)
    #include <memory>
    #include <wx/string.h>
    #include <wx/object.h>
#elif defined(NB_FRAMEWORK_QT)
    #include <memory>
    #include <QString>
    #include <QObject>
#else
    #include <memory>
    #include <string>
#endif
#include <sstream>

namespace NextBiometrics
{
    namespace System
    {
        #include <NBTypes.h>

        #if !defined(NB_CPP)
           #error "Only C++ is supported"
        #endif

        #undef NBTrue
        #undef NBFalse

        const NBBool NBTrue = 1;
        const NBBool NBFalse = 0;

#if defined(NB_FRAMEWORK_MFC)
        typedef CStringA NBString;
#elif defined (NB_FRAMEWORK_WX)
        typedef wxString NBString;
#elif defined(NB_FRAMEWORK_QT)
        typedef QString NBString;
#else
        typedef std::string NBString;
#endif

        __inline NBString ToNBString(const NBAChar * szValue, NBUInt uiValueLength)
        {
#if defined(NB_FRAMEWORK_MFC)
            return CStringA(szValue, uiValueLength);
#elif defined(NB_FRAMEWORK_WX)
            return wxString(szValue, wxConvLibc, uiValueLength);
#elif defined(NB_FRAMEWORK_QT)
            return QString::fromLocal8Bit(szValue, uiValueLength);
#else
            return std::string(szValue, uiValueLength);
#endif
        }

        __inline const NBAChar * FromNBString(const NBString& szValue)
        {
#if defined(NB_FRAMEWORK_MFC)
            return (const NBAChar *)szValue;
#elif defined(NB_FRAMEWORK_WX)
            return szValue.c_str();
#elif defined(NB_FRAMEWORK_QT)
            return szValue.toStdString().c_str();
#else
            return szValue.c_str();
#endif
        }

        __inline int NBStoi(const std::string& value)
        {
            int i;
            std::istringstream(value) >> i;
            return i;
        }

        __inline long NBStol(const std::string& value)
        {
            long l;
            std::istringstream(value) >> l;
            return l;
        }

        __inline unsigned long NBStoul(const std::string& value)
        {
            unsigned long l;
            std::istringstream(value) >> l;
            return l;
        }

        __inline unsigned long long NBStoull(const std::string& value)
        {
            unsigned long long l;
            std::istringstream(value) >> l;
            return l;
        }

        __inline std::string NBToString(long l)
        {
            std::ostringstream s;
            s << l;
            return s.str();
        }

        class EventArgs
        {
            void * pParam;
        public:
            EventArgs(void * pParam)
            {
                this->pParam = pParam;
            }

            void * GetParam() const
            {
                return pParam;
            }
        };
    }
}

#endif // !NB_TYPES_HPP
