﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using NBBiometricsSampleCS.Misc;
using NextBiometrics.Biometrics;

namespace NBBiometricsSampleCS.Forms
{
    public partial class EnrollForm : Form
    {
        public EnrollForm()
        {
            InitializeComponent();
        }

        public NBBiometricsContext BiometricsContext { get; set; }
        public NBBiometricsTemplate Template { get; set; }
        public Database Database { get; set; }

        private void EnrollForm_Load(object sender, EventArgs e)
        {
            if (BiometricsContext != null && Template != null)
            {
                var convertionTypes = new List<TemplateTypeInfo>();
                var templateType = BiometricsContext.SupportedTemplateTypes.First(x => x.TemplateType == Template.Type);
                convertionTypes.Add(new TemplateTypeInfo(templateType));
                if (templateType.IsConversionFromSupported)
                {
                    var templateTypes = BiometricsContext.SupportedTemplateTypes.Where(x =>
                        x.TemplateType != templateType.TemplateType && x.IsConversionToSupported && (x.IsVerificationSupported || x.IsIdentificationSupported));
                    convertionTypes.AddRange(templateTypes.Select(x => new TemplateTypeInfo(x)));
                }
                comboBoxConvertionTemplate.Items.AddRange(convertionTypes.ToArray());
                if (comboBoxConvertionTemplate.Items.Count > 0)
                    comboBoxConvertionTemplate.SelectedIndex = 0;
            }
        }

        private void EnrollForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (DialogResult == DialogResult.OK)
            {
                try
                {
                    string name = textBoxName.Text;
                    if (string.IsNullOrEmpty(name))
                    {
                        e.Cancel = true;
                        MessageBox.Show(this, "Name is empty. Please add proper name and try again.",
                            "Error information", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        return;
                    }

                    var convertedTemplate =
                        ConvertTemplate(comboBoxConvertionTemplate.SelectedItem as TemplateTypeInfo);
                    Database.Add(name, convertedTemplate);
                    MessageBox.Show(this, "Template enrolled!", "Information", MessageBoxButtons.OK,
                        MessageBoxIcon.Information);
                }
                catch (Exception ex)
                {
                    e.Cancel = true;
                    MessageBox.Show(this, string.Format("Failed to add to database. Reason: {0}", ex.Message),
                        "Error information", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void buttonSave_Click(object sender, EventArgs e)
        {
            TemplateUtils.SaveTemplate(BiometricsContext.SaveTemplate(ConvertTemplate(comboBoxConvertionTemplate.SelectedItem as TemplateTypeInfo)));
        }

        private NBBiometricsTemplate ConvertTemplate(TemplateTypeInfo destinationType)
        {
            if (destinationType == null || destinationType.Type.TemplateType == Template.Type)
            {
                return Template;
            }

            return BiometricsContext.ConvertTemplate(Template, destinationType.Type.TemplateType);
        }
    }
}
