﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using NBBiometricsSampleCS.Misc;
using NextBiometrics.Biometrics;

namespace NBBiometricsSampleCS.Forms
{
    public partial class ExportForm : Form
    {
        public ExportForm()
        {
            InitializeComponent();
        }

        public NBBiometricsContext BiometricsContext { get; set; }
        public NBBiometricsTemplate Template { get; set; }
        public Database Database { get; set; }

        private void ExportForm_Load(object sender, EventArgs e)
        {
            if (BiometricsContext != null && Template != null)
            {
                var convertionTypes = new List<TemplateTypeInfo>();
                var templateType = BiometricsContext.SupportedTemplateTypes.First(x => x.TemplateType == Template.Type);
                convertionTypes.Add(new TemplateTypeInfo(templateType));
                if (templateType.IsConversionFromSupported)
                {
                    var templateTypes = BiometricsContext.SupportedTemplateTypes.Where(x =>
                        x.TemplateType != templateType.TemplateType && x.IsConversionToSupported);
                    convertionTypes.AddRange(templateTypes.Select(x => new TemplateTypeInfo(x)));
                }
                comboBoxConvertionTemplate.Items.AddRange(convertionTypes.ToArray());
                if (comboBoxConvertionTemplate.Items.Count > 0)
                    comboBoxConvertionTemplate.SelectedIndex = 0;
            }
        }

        private void ExportForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (DialogResult == DialogResult.OK)
            {
                try
                {
                    TemplateUtils.SaveTemplate(BiometricsContext.SaveTemplate(ConvertTemplate(comboBoxConvertionTemplate.SelectedItem as TemplateTypeInfo)));
                }
                catch (Exception ex)
                {
                    e.Cancel = true;
                    MessageBox.Show(this, string.Format("Failed to export template. Reason: {0}", ex.Message),
                        "Error information", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private NBBiometricsTemplate ConvertTemplate(TemplateTypeInfo destinationType)
        {
            if (destinationType == null || destinationType.Type.TemplateType == Template.Type)
            {
                return Template;
            }

            return BiometricsContext.ConvertTemplate(Template, destinationType.Type.TemplateType);
        }
    }
}
