﻿using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using NextBiometrics.Devices;

namespace NBBiometricsSampleCS.Misc
{
    public static class ImageUtils
    {
        public static Bitmap CreateImage(NBDeviceScanFormatInfo format, byte[] image)
        {
            Bitmap bmp = new Bitmap((int)format.Width, (int)format.Height, PixelFormat.Format8bppIndexed);
            BitmapData bmpData = bmp.LockBits(new Rectangle(0, 0, (int)format.Width, (int)format.Height),
                ImageLockMode.WriteOnly, PixelFormat.Format8bppIndexed);
            int stride = bmpData.Stride;
            int offset = stride - (int)format.Width;
            IntPtr iptr = bmpData.Scan0;
            int scanBytes = stride * (int)format.Height;
            int posScan = 0, posReal = 0;
            byte[] pixelValues = new byte[scanBytes];
            for (int x = 0; x < format.Height; x++)
            {
                for (int y = 0; y < format.Width; y++)
                {
                    pixelValues[posScan++] = image[posReal++];
                }
                posScan += offset;
            }
            Marshal.Copy(pixelValues, 0, iptr, scanBytes);
            bmp.UnlockBits(bmpData);
            ColorPalette tempPalette;
            using (Bitmap tempBmp = new Bitmap(1, 1, PixelFormat.Format8bppIndexed))
            {
                tempPalette = tempBmp.Palette;
            }
            for (int i = 0; i < 256; i++)
            {
                tempPalette.Entries[i] = Color.FromArgb(i, i, i);
            }
            bmp.Palette = tempPalette;

            bmp.SetResolution(format.HorizontalResolution, format.VerticalResolution);

            return bmp;
        }

        public static void SaveImage(Image image)
        {
            if (image != null)
            {
                SaveFileDialog saveImageFileDialog = new SaveFileDialog
                {
                    Filter = @"Bitmap Image|*.bmp|GIF Image|*.gif|JPEG Image|*.jpg|PNG Image|*.png",
                    Title = @"Save Image File"
                };
                saveImageFileDialog.ShowDialog();

                if (!string.IsNullOrEmpty(saveImageFileDialog.FileName))
                {
                    using (FileStream fs = (FileStream)saveImageFileDialog.OpenFile())
                    {
                        switch (saveImageFileDialog.FilterIndex)
                        {
                            case 1:
                                image.Save(fs, ImageFormat.Bmp);
                                break;
                            case 2:
                                image.Save(fs, ImageFormat.Gif);
                                break;
                            case 3:
                                image.Save(fs, ImageFormat.Jpeg);
                                break;
                            case 4:
                                image.Save(fs, ImageFormat.Png);
                                break;
                        }
                    }
                }
            }
        }
    }
}
