﻿using System;
using System.Windows.Forms;
using NBBiometricsSampleCS.Misc;
using NextBiometrics.Biometrics;
using NextBiometrics.Devices;

namespace NBBiometricsSampleCS.Tabs
{
    public partial class BaseTab : UserControl
    {
        #region Private fields

        private NBDevice _device;
        private NBBiometricsContext _biometricsContext;
        private Database _database;
        private bool _isBusy;

        #endregion

        #region Protected constructor

        protected BaseTab()
        {
            InitializeComponent();
            IsBusy = false;
        }

        #endregion

        #region Public properties

        public virtual NBDevice Device
        {
            get { return _device; }
            set { _device = value; }
        }

        public virtual NBBiometricsContext BiometricsContext
        {
            get { return _biometricsContext; }
            set { _biometricsContext = value; }
        }

        public virtual Database Database
        {
            get { return _database; }
            set { _database = value; }
        }

        public virtual bool IsBusy
        {
            get { return _isBusy; }
            protected set { _isBusy = value; }
        }

        public event EventHandler BusyChanged;

        #endregion

        #region Public methods

        public virtual void RefreshData()
        {
        }

        public virtual void Cancel()
        {
        }

        #endregion

        #region Protected methods

        protected void OnBusyChanged()
        {
            if (BusyChanged != null)
            {
                BusyChanged.Invoke(this, EventArgs.Empty);
            }
        }

        #endregion
    }
}
