﻿using System;
using System.IO;
using System.Windows.Forms;
using NBBiometricsSampleCS.Forms;
using NBBiometricsSampleCS.Misc;
using NextBiometrics.Biometrics;

namespace NBBiometricsSampleCS.Tabs
{
    public partial class DatabaseTab : BaseTab
    {
        #region Public constructor

        public DatabaseTab()
        {
            InitializeComponent();
        }

        #endregion

        #region Private methods

        private void ClearInfo()
        {
            listBoxDatabase.Items.Clear();
            ClearInfoData();
        }

        private void ClearInfoData()
        {
            groupBox.Enabled = false;
            textBoxName.Text = textBoxPosition.Text = textBoxTemplateType.Text = string.Empty;
        }

        private void SetItem(Database.Template template)
        {
            textBoxName.Text = template.Name;
            textBoxPosition.Text = template.Data.Position.ToString();
            textBoxTemplateType.Text = template.Data.Type.ToString();
        }

        private NBBiometricsTemplate LoadTemplate(NBBiometricsTemplateType templateType, byte[] data)
        {
            try
            {
                return BiometricsContext.LoadTemplate(templateType, data);
            }
            catch (Exception)
            {
                return null;
            }
        }

        private void ImportItem(string data)
        {
            try
            {
                var fileData = File.ReadAllBytes(data);
                var templateInfos = BiometricsContext.SupportedTemplateTypes;
                NBBiometricsTemplate template = null;
                foreach (var templateInfo in templateInfos)
                {
                    template = LoadTemplate(templateInfo.TemplateType, fileData);
                    if (template != null) break;
                }
                if (template == null)
                    throw new ApplicationException("Please check if it is of supported format and try again.");
                var fileName = Path.GetFileNameWithoutExtension(data);
                Database.Add(fileName, template);
                MessageBox.Show(this, "Template imported!", "Information", MessageBoxButtons.OK,
                    MessageBoxIcon.Information);
            }
            catch (Exception e)
            {
                MessageBox.Show(this, string.Format("Failed to import template. Reason: {0}.", e.Message),
                    "Error information", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        #endregion

        #region Public methods

        public override void RefreshData()
        {
            ClearInfo();
            if (Database != null)
            {
                foreach (var item in Database.Templates)
                {
                    listBoxDatabase.Items.Add(item);
                }
            }
        }

        #endregion

        #region Private form events

        private void listBoxDatabase_SelectedIndexChanged(object sender, EventArgs e)
        {
            ClearInfoData();
            var selectedItem = listBoxDatabase.SelectedItem as Database.Template;
            if (selectedItem != null)
            {
                SetItem(selectedItem);
                groupBox.Enabled = true;
            }
        }

        private void buttonSave_Click(object sender, EventArgs e)
        {
            var selectedItem = listBoxDatabase.SelectedItem as Database.Template;
            if (selectedItem != null)
            {
                using (var exportForm = new ExportForm())
                {
                    exportForm.BiometricsContext = BiometricsContext;
                    exportForm.Database = Database;
                    exportForm.Template = selectedItem.Data;
                    exportForm.ShowDialog();
                }
            }
        }

        private void buttonDelete_Click(object sender, EventArgs e)
        {
            var selectedItem = listBoxDatabase.SelectedItem as Database.Template;
            if (selectedItem != null)
            {
                Database.Remove(selectedItem);
                RefreshData();
            }
        }

        private void buttonImport_Click(object sender, EventArgs e)
        {
            using (var openDialog = new OpenFileDialog())
            {
                if (openDialog.ShowDialog() == DialogResult.OK)
                {
                    ImportItem(openDialog.FileName);
                    RefreshData();
                }
            }
        }

        #endregion
    }
}
